# Ridiculously simple test of the os.startfile function for Windows.
#
# empty.vbs is an empty file (except for a comment), which does
# nothing when run with cscript or wscript.
#
# A possible improvement would be to have empty.vbs do something that
# we can detect here, to make sure that not only the os.startfile()
# call succeeded, but also the the script actually has run.

import unittest
from test import test_support
import os
from os import path
from time import sleep

startfile = test_support.get_attribute(os, 'startfile')


class TestCase(unittest.TestCase):
    def test_nonexisting(self):
        self.assertRaises(OSError, startfile, "nonexisting.vbs")

    def test_nonexisting_u(self):
        self.assertRaises(OSError, startfile, u"nonexisting.vbs")

    def test_empty(self):
        empty = path.join(path.dirname(__file__), "empty.vbs")
        startfile(empty)
        startfile(empty, "open")
        # Give the child process some time to exit before we finish.
        # Otherwise the cleanup code will not be able to delete the cwd,
        # because it is still in use.
        sleep(0.1)

    def test_empty_u(self):
        empty = path.join(path.dirname(__file__), "empty.vbs")
        startfile(unicode(empty, "mbcs"))
        startfile(unicode(empty, "mbcs"), "open")
        sleep(0.1)

def test_main():
    test_support.run_unittest(TestCase)

if __name__=="__main__":
    test_main()



"""Tests for json.

The tests for json are defined in the json.tests package;
the test_suite() function there returns a test suite that's ready to
be run.
"""

import json.tests
import test.test_support


def test_main():
    test.test_support.run_unittest(json.tests.test_suite())


if __name__ == "__main__":
    test_main()



"""Basic tests for os.popen()

  Particularly useful for platforms that fake popen.
"""

import unittest
from test import test_support
import os, sys

# Test that command-lines get down as we expect.
# To do this we execute:
#    python -c "import sys;print sys.argv" {rest_of_commandline}
# This results in Python being spawned and printing the sys.argv list.
# We can then eval() the result of this, and see what each argv was.
python = sys.executable

class PopenTest(unittest.TestCase):
    def _do_test_commandline(self, cmdline, expected):
        cmd = '%s -c "import sys;print sys.argv" %s' % (python, cmdline)
        data = os.popen(cmd).read() + '\n'
        got = eval(data)[1:] # strip off argv[0]
        self.assertEqual(got, expected)

    def test_popen(self):
        self.assertRaises(TypeError, os.popen)
        self._do_test_commandline(
            "foo bar",
            ["foo", "bar"]
        )
        self._do_test_commandline(
            'foo "spam and eggs" "silly walk"',
            ["foo", "spam and eggs", "silly walk"]
        )
        self._do_test_commandline(
            'foo "a \\"quoted\\" arg" bar',
            ["foo", 'a "quoted" arg', "bar"]
        )
        test_support.reap_children()

    def test_return_code(self):
        self.assertEqual(os.popen("exit 0").close(), None)
        if os.name == 'nt':
            self.assertEqual(os.popen("exit 42").close(), 42)
        else:
            self.assertEqual(os.popen("exit 42").close(), 42 << 8)

def test_main():
    test_support.run_unittest(PopenTest)

if __name__ == "__main__":
    test_main()



from ctypes import *
import sys, unittest
import os
from ctypes.util import find_library
from ctypes.test import is_resource_enabled

libc_name = None
if os.name == "nt":
    libc_name = find_library("c")
elif os.name == "ce":
    libc_name = "coredll"
elif sys.platform == "cygwin":
    libc_name = "cygwin1.dll"
else:
    libc_name = find_library("c")

if is_resource_enabled("printing"):
    print "libc_name is", libc_name

class LoaderTest(unittest.TestCase):

    unknowndll = "xxrandomnamexx"

    @unittest.skipUnless(libc_name is not None, 'could not find libc')
    def test_load(self):
        CDLL(libc_name)
        CDLL(os.path.basename(libc_name))
        self.assertRaises(OSError, CDLL, self.unknowndll)

    @unittest.skipUnless(libc_name is not None, 'could not find libc')
    @unittest.skipUnless(libc_name is not None and
                         os.path.basename(libc_name) == "libc.so.6",
                         'wrong libc path for test')
    def test_load_version(self):
        cdll.LoadLibrary("libc.so.6")
        # linux uses version, libc 9 should not exist
        self.assertRaises(OSError, cdll.LoadLibrary, "libc.so.9")
        self.assertRaises(OSError, cdll.LoadLibrary, self.unknowndll)

    def test_find(self):
        for name in ("c", "m"):
            lib = find_library(name)
            if lib:
                cdll.LoadLibrary(lib)
                CDLL(lib)

    @unittest.skipUnless(os.name in ("nt", "ce"),
                         'test specific to Windows (NT/CE)')
    def test_load_library(self):
        self.assertIsNotNone(libc_name)
        if is_resource_enabled("printing"):
            print find_library("kernel32")
            print find_library("user32")

        if os.name == "nt":
            windll.kernel32.GetModuleHandleW
            windll["kernel32"].GetModuleHandleW
            windll.LoadLibrary("kernel32").GetModuleHandleW
            WinDLL("kernel32").GetModuleHandleW
        elif os.name == "ce":
            windll.coredll.GetModuleHandleW
            windll["coredll"].GetModuleHandleW
            windll.LoadLibrary("coredll").GetModuleHandleW
            WinDLL("coredll").GetModuleHandleW

    @unittest.skipUnless(os.name in ("nt", "ce"),
                         'test specific to Windows (NT/CE)')
    def test_load_ordinal_functions(self):
        import _ctypes_test
        dll = WinDLL(_ctypes_test.__file__)
        # We load the same function both via ordinal and name
        func_ord = dll[2]
        func_name = dll.GetString
        # addressof gets the address where the function pointer is stored
        a_ord = addressof(func_ord)
        a_name = addressof(func_name)
        f_ord_addr = c_void_p.from_address(a_ord).value
        f_name_addr = c_void_p.from_address(a_name).value
        self.assertEqual(hex(f_ord_addr), hex(f_name_addr))

        self.assertRaises(AttributeError, dll.__getitem__, 1234)

    @unittest.skipUnless(os.name == "nt", 'Windows-specific test')
    def test_1703286_A(self):
        from _ctypes import LoadLibrary, FreeLibrary
        # On winXP 64-bit, advapi32 loads at an address that does
        # NOT fit into a 32-bit integer.  FreeLibrary must be able
        # to accept this address.

        # These are tests for http://www.python.org/sf/1703286
        handle = LoadLibrary("advapi32")
        FreeLibrary(handle)

    @unittest.skipUnless(os.name == "nt", 'Windows-specific test')
    def test_1703286_B(self):
        # Since on winXP 64-bit advapi32 loads like described
        # above, the (arbitrarily selected) CloseEventLog function
        # also has a high address.  'call_function' should accept
        # addresses so large.
        from _ctypes import call_function
        advapi32 = windll.advapi32
        # Calling CloseEventLog with a NULL argument should fail,
        # but the call should not segfault or so.
        self.assertEqual(0, advapi32.CloseEventLog(None))
        windll.kernel32.GetProcAddress.argtypes = c_void_p, c_char_p
        windll.kernel32.GetProcAddress.restype = c_void_p
        proc = windll.kernel32.GetProcAddress(advapi32._handle,
                                              "CloseEventLog")
        self.assertTrue(proc)
        # This is the real test: call the function via 'call_function'
        self.assertEqual(0, call_function(proc, (None,)))

if __name__ == "__main__":
    unittest.main()



import os
import re
from functools import wraps

import sys

from whatap import check_whatap_home, ROOT_DIR, init_config, update_config, \
    batch_agent, AGENT_NAME


def whatap_command(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        if len(sys.argv) == 1:
            print('WHATAP: INVALID ARGMENTS.')
            print(func.__doc__)
        else:
            if check_whatap_home():
                return func(*args, **kwargs)
    
    return wrapper

def start_agent_batch():
    batch_agent()

def stop_agent():
    try:
        os.system('killall {}'.format(AGENT_NAME))
    except Exception, e:
        print(
            'Try to execute command. \n  {}'.format(
                '`ps -ef | grep {}`'.format(AGENT_NAME)))
        print(
            'Next, `sudo killall {}`'.format(AGENT_NAME))

    print('WHATAP: AGENT STOP.')
    
@whatap_command
def start_agent():
    """
    Usage:
         whatap-start-agent [YOUR_APPLICATION_START_COMMAND]
    """
    
    args = sys.argv[1:]
    root_directory = os.path.dirname(ROOT_DIR)
    boot_directory = os.path.join(root_directory, 'bootstrap')
    
    python_path = boot_directory
    
    if 'PYTHONPATH' in os.environ:
        path = os.environ['PYTHONPATH'].split(os.path.pathsep)
        if not boot_directory in path:
            python_path = "%s%s%s" % (boot_directory, os.path.pathsep,
                                      os.environ['PYTHONPATH'])
    
    os.environ['PYTHONPATH'] = python_path
    
    program_exe_path = args[0]
    if not os.path.dirname(program_exe_path):
        program_search_path = os.environ.get('PATH', '').split(os.path.pathsep)
        for path in program_search_path:
            path = os.path.join(path, program_exe_path)
            if os.path.exists(path) and os.access(path, os.X_OK):
                program_exe_path = path
                break
    
    try:
        os.execl(program_exe_path, *args)
    except Exception, e:
        print('WHATAP: INVALID ARGMENTS.')
        print(start_agent.__doc__)


OPTIONS = ['--host', '--license', '--app_name', '--app_process_name']


@whatap_command
def setting_config():
    """
    Usage:
        whatap-setting-config [--host HOST_ADDR]
                              [--license LICENSE_KEY]
                              [--app_name APPLICATION_NAME]
                              [--app_process_name APPLICATION_PROCESS_NAME]
    """
    home = 'WHATAP_HOME'
    if init_config(home):
        print('WHATAP:')
        args = ' '.join(sys.argv[1:])
        is_success=False
        for opt in OPTIONS:
            key = opt.split('--')[1]
            m = re.match(r"[\s\w\-\_\.\/]*" + key + " (?P<key>[\w\-\_\.\/]*)", args)
            if m:
                if key == 'host':
                    key = 'whatap.server.{}'.format(key)
                update_config(home, key, m.group('key'))
                print('\t{}={}'.format(key, m.group('key')))
                is_success = True
                
        if is_success:
            print('SUCCESS!')
        else:
            print(setting_config.__doc__)
    
    



import whatap.net as net
import whatap.agent.conf.configure as config
import whatap.agent.secure.security_master as secure
import threading, time, socket
from whatap.io.data_inputx import DataInputX
from whatap.io.data_outputx import DataOutputX
import whatap.util.logging_util as logging_util
from whatap.util.date_util import DateUtil as dateutil
from datetime import datetime, timedelta
import platform, subprocess

READ_MAX = 8 * 1024 * 1024
moduleLoaded = datetime.now()

class TcpSession :

    def __init__(self):
        self.client = None
        self.in_ = None
        self.dest = 0
        self.hosts = None
        self.started = 0
        self.lastReceived = datetime(year=1970, month=1, day=1)

    def isHostChanged(self):
        conf = config.GetConfig()
        hosts = conf.WhatapHost.split('/')
        if hosts is not None and len(hosts) > 0 :
            ismatch = False
            if self.hosts is not None and len(self.hosts) > 0 :
                for h1 in hosts :
                    for h2 in self.hosts :
                        if h1 == h2 :
                            ismatch = True

            if not ismatch :
                return True

        return False

    def open(self):
        try:
            return self.__open()
        except Exception, e:
            logging_util.error(str(e))
            self.Close()
            return False

    def __open(self):
        if self.isOpen() :
            return True

        pcode = secure.GetSecurityMaster().pcode
        if pcode == 0 :
            logging_util.error("[TcpSession] Pcode Error")
            return False

        conf = config.GetConfig()
        hosts = conf.WhatapHost.split('/')
        if hosts is None or len(hosts) == 0 :
            logging_util.error("[TcpSession] whatap host config Error")
            self.Close()
            return False
        if self.dest >= len(hosts) :
            self.dest = 0
        port = int(conf.WhatapPort)
        client = socket.socket( socket.AF_INET, socket.SOCK_STREAM)
        client.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
        try:
            client.connect((hosts[self.dest], port))
        except Exception, e:
            logging_util.error("[TcpSession] tcp connect exception : ", e)
            self.dest += 1
            self.Close()
            return False

        self.hosts = hosts
        myIP = client.getsockname()[0]
        if myIP == 0 and 'hp-ux' == platform.system().lower(): # HPUX pa not support getsockname
            command = "netstat -an | grep 6600 | grep ESTABLISHED | head -n 1 | awk '{print $4}' | awk -F'.' '{print $1\".\"$2\".\"$3\".\"$4}'"

            try:
                netstat = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
                myIP = netstat.stdout.read().strip('\n')
            except subprocess.CalledProcessError as e:
                logging_util.error("netstat error", e)
                
        secure.GetSecurityMaster().DecideAgentOnameOid(myIP)


        keyReset = self.keyReset()

        client.sendall(keyReset)
        self.in_ = DataInputX(client)
        data = self.readKeyReset(self.in_)
        if data is None or len(data) < 1 :
            logging_util.error("[TcpSession] key read error")
            return False
        secure.UpdateNetCypherKey(data)
        self.client = client
        self.started = datetime.now()

        return True

    def isOpen(self):
        return self.client is not None

    def readKeyReset(self, in_):
        r = in_.readByte()
        r = in_.readByte()
        pcode = in_.readLong()
        oid = in_.readInt()
        r = in_.readInt()
        data = in_.readIntBytes(1024)
        if data is None:
            logging_util.error("[TcpSession] key reset error")
            return None

        secu = secure.GetSecurityMaster()

        #print 'readKeyReset ', pcode, secu.pcode, 'oid', oid, secu.oid
        if pcode != secu.pcode or oid != secu.oid :
            logging_util.error("[TcpSession] pcode, oid error")
            return None
        else:
            return data

    def keyReset(self):
        conf = config.GetConfig()
        secu = secure.GetSecurityMaster()
        secu.WaitForInit()
        dout = DataOutputX()

        msg = dout.writeText("hello").writeText(secu.oname).writeInt(secu.ip).toByteArray()
        if conf.CypherLevel > 0 :
            msg = secu.cypher.Encrypt(msg)
        dout = DataOutputX()
        dout.writeByte(net.NETSRC_AGENT_JAVA_EMBED)

        trkey = 0
        if conf.CypherLevel == 128 :
            dout.writeByte(net.NET_KEY_RESET)
        else:
            dout.writeByte(net.NET_KEY_EXTENSION)

            if conf.CypherLevel == 0 :
                trkey = 0
            else :
                b0 = 1
                b1 = conf.CypherLevel / 8
                trkey = DataOutputX.toInt([b0, b1, 0, 0], 0)

        #print 'keyReset', secu.pcode, secu.oid
        dout.writeLong(secu.pcode)
        dout.writeInt(secu.oid)
        dout.writeInt(trkey)
        dout.writeIntBytes(msg)

        return dout.toByteArray()

    def Send(self, code , b ):
        try:
            return self.__send(code, b)
        except:
            self.Close()

    def __send(self, code, b):
        if self.client is None :
            return False

        secu = secure.GetSecurityMaster()
        secuSession = secure.GetSecuritySession()
        out = DataOutputX()
        out.writeByte(net.NETSRC_AGENT_JAVA_EMBED)
        out.writeByte(code)
        out.writeLong(secu.pcode)
        out.writeInt(secu.oid)
        out.writeInt(secuSession.transferKey)
        if hasattr(b, 'getvalue'):
            out.writeIntBytes(b.getvalue())
        else:
            out.writeIntBytes(b)
        sendbuf = out.toByteArray()
        self.client.sendall(sendbuf)
        return True

    def Close(self):
        if self.client is not None:
            self.client.close()
        self.client = None

    def Read(self):
        if not self.isOpen():
            return empty

        read_start = datetime.now()
        step = 'src'
        code = None
        pcode = None
        oid = None
        transfer_key = None
        data_len = -1
        try:
            self.in_.readByte()
            step = 'code'
            code = self.in_.readByte()
            step = 'pcode'
            pcode = self.in_.readLong()
            step = 'oid'
            oid = self.in_.readInt()
            step = 'transferKey'
            transfer_key = self.in_.readInt()
            step = 'data'
            data = self.in_.readIntBytes(READ_MAX)
            data_len = -1 if data is None else len(data)
        except Exception, e:
            elapsed_ms = int((datetime.now() - read_start).total_seconds() * 1000)
            logging_util.error(
                "[TcpSession] Read failed step=%s elapsed=%dms code=%s pcode=%s oid=%s trkey=%s err=%s" %
                (step, elapsed_ms, code, pcode, oid, transfer_key, e))
            raise

        if data is None :
            logging_util.error("[TcpSession] Read got None data code=%s" % code)
            return empty

        secu = secure.GetSecurityMaster()
        if pcode != secu.pcode or oid != secu.oid :
            logging_util.error(
                "[TcpSession] Read pcode/oid mismatch recv(pcode=%s,oid=%s) expect(pcode=%s,oid=%s) code=%s len=%d" %
                (pcode, oid, secu.pcode, secu.oid, code, data_len))
            return empty

        self.lastReceived = datetime.now()
        return TcpReturn(code= code, data= data, transfer_key= transfer_key)

class TcpReturn:
    def __init__(self, code=None, data=None, transfer_key = None):
        self.code = code
        self.data = data
        self.transferKey = transfer_key

def connect(s):
    while True:
        while not s.open():
            logging_util.error("[TcpSession] Session Open Error")
            time.sleep(3)
        now = dateutil.now()
        if not s.Send(net.NET_TIME_SYNC, DataOutputX.toBytesLong(now)):
            logging_util.error("[TcpSession] TimeSync Error")
            s.Close()
            time.sleep(3)
            continue
        break

def WaitForConnection():
    while True:
        tcp = GetTcpSession()
        if tcp.isOpen() :
            break

        time.sleep(1)

empty = TcpReturn()
sessionLock = threading.Lock()
session = None
def GetTcpSession():
    global session
    sessionLock.acquire()
    try:
        if session and session.isOpen() and not session.isHostChanged():
            return session
        if session :
            logging_util.info("[TcpSession] Tcp Session Reconnect")
            if not session.isOpen():
                logging_util.error("[TcpSession] Session Closed")
            if session.isHostChanged():
                logging_util.info("[TcpSession] Whatap Config Host Option Changed")
            session.Close()
        session = TcpSession()
        connect(session)
        logging_util.info("[TcpSession] Connection : ", session.hosts)
        return session
    finally:
        sessionLock.release()

def isSessionOK():
    ret = True

    now = datetime.now()
    if now-moduleLoaded > timedelta(minutes=config.GetConfig().HealthCheckCoolTime) :
        if not session:
            return False

        ret = session.isOpen()
        ret = ret and (session.started is not 0)

        if session.started and now-session.started > timedelta(minutes=config.GetConfig().HealthCheckCoolTime):
            ret = ret and (now-session.lastReceived < timedelta(seconds=config.GetConfig().TimeSyncInterval*10))

    return ret

def net_test(connect_timeout=5, response_timeout=3):
    secu = secure.GetSecurityMaster()
    secu.WaitForInitFor(30)

    if secu.pcode == 0:
        return "invalid license error"

    conf = config.GetConfig()
    hosts = conf.WhatapHost.split('/')
    if hosts is None or len(hosts) == 0:
        return "whatap host config error"

    port = int(conf.WhatapPort)
    client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    client.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)

    connected = False
    for host in hosts:
        try:
            client.connect((host.strip(), port))
            connected = True
            break
        except Exception, e:
            logging_util.error("[NetTest] connect failed to %s:%s - %s" % (host, port, e))
            continue

    if not connected:
        client.close()
        return "connection failed - host: %s, port: %s" % (conf.WhatapHost, port)

    test_session = TcpSession()
    test_session.client = client
    try:
        test_session.hosts = hosts
        myIP = client.getsockname()[0]
        secu.DecideAgentOnameOid(myIP)

        keyReset = test_session.keyReset()
        client.sendall(keyReset)
        test_session.in_ = DataInputX(client)
        data = test_session.readKeyReset(test_session.in_)
        if data is None or len(data) < 1:
            return "key reset failed"
        secure.UpdateNetCypherKey(data)

        now = dateutil.now()
        if not test_session.Send(net.NET_TIME_SYNC, DataOutputX.toBytesLong(now)):
            return "send failed - host: %s, port: %s" % (conf.WhatapHost, port)

        is_response_received = [False]

        def read_response():
            try:
                client.settimeout(response_timeout)
                out = test_session.Read()
                if out and out.code == net.NET_TIME_SYNC:
                    is_response_received[0] = True
            except Exception, e:
                logging_util.error("[NetTest] read response error: %s" % e)

        reader = threading.Thread(target=read_response)
        reader.setDaemon(True)
        reader.start()
        reader.join(response_timeout + 1)

        if not is_response_received[0]:
            return "received no response - host: %s, port: %s" % (conf.WhatapHost, port)

        return None

    except Exception, e:
        return "net test error: %s" % str(e)
    finally:
        test_session.Close()



try:
    import idlelib.PyShell
except ImportError:
    # IDLE is not installed, but maybe PyShell is on sys.path:
    try:
        import PyShell
    except ImportError:
        raise
    else:
        import os
        idledir = os.path.dirname(os.path.abspath(PyShell.__file__))
        if idledir != os.getcwd():
            # We're not in the IDLE directory, help the subprocess find run.py
            pypath = os.environ.get('PYTHONPATH', '')
            if pypath:
                os.environ['PYTHONPATH'] = pypath + ':' + idledir
            else:
                os.environ['PYTHONPATH'] = idledir
        PyShell.main()
else:
    idlelib.PyShell.main()



import os.path
import sys

# If we are working on a development version of IDLE, we need to prepend the
# parent of this idlelib dir to sys.path.  Otherwise, importing idlelib gets
# the version installed with the Python used to call this module:
idlelib_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, idlelib_dir)

import idlelib.PyShell
idlelib.PyShell.main()



"""Test cases for the fnmatch module."""

from test import test_support
import unittest

from fnmatch import fnmatch, fnmatchcase, _MAXCACHE, _cache
from fnmatch import fnmatch, fnmatchcase, _MAXCACHE, _cache, _purge


class FnmatchTestCase(unittest.TestCase):

    def tearDown(self):
        _purge()

    def check_match(self, filename, pattern, should_match=1, fn=fnmatch):
        if should_match:
            self.assertTrue(fn(filename, pattern),
                         "expected %r to match pattern %r"
                         % (filename, pattern))
        else:
            self.assertTrue(not fn(filename, pattern),
                         "expected %r not to match pattern %r"
                         % (filename, pattern))

    def test_fnmatch(self):
        check = self.check_match
        check('abc', 'abc')
        check('abc', '?*?')
        check('abc', '???*')
        check('abc', '*???')
        check('abc', '???')
        check('abc', '*')
        check('abc', 'ab[cd]')
        check('abc', 'ab[!de]')
        check('abc', 'ab[de]', 0)
        check('a', '??', 0)
        check('a', 'b', 0)

        # these test that '\' is handled correctly in character sets;
        # see SF bug #409651
        check('\\', r'[\]')
        check('a', r'[!\]')
        check('\\', r'[!\]', 0)

        # test that filenames with newlines in them are handled correctly.
        # http://bugs.python.org/issue6665
        check('foo\nbar', 'foo*')
        check('foo\nbar\n', 'foo*')
        check('\nfoo', 'foo*', False)
        check('\n', '*')

    def test_fnmatchcase(self):
        check = self.check_match
        check('AbC', 'abc', 0, fnmatchcase)
        check('abc', 'AbC', 0, fnmatchcase)

    def test_cache_clearing(self):
        # check that caches do not grow too large
        # http://bugs.python.org/issue7846

        # string pattern cache
        for i in range(_MAXCACHE + 1):
            fnmatch('foo', '?' * i)

        self.assertLessEqual(len(_cache), _MAXCACHE)

def test_main():
    test_support.run_unittest(FnmatchTestCase)


if __name__ == "__main__":
    test_main()



import subprocess
import functools
import os
import sys
import re

from collections import namedtuple

from . import _common
from ._common import get_procfs_path
from ._common import memoize
from ._common import memoize_when_activated
from ._common import NoSuchProcess
from ._common import ZombieProcess
from ._compat import FileNotFoundError
from ._compat import PermissionError
from ._compat import ProcessLookupError
from ._compat import which
from . import _psutil_hpux as cext


__extra__all__ = []

sdiskio = namedtuple(
        'sdiskio', ['read_count', 'write_count',
            'read_bytes', 'write_bytes',
            'read_time', 'write_time',
            'busy_time'])


#


#

def boot_time():
    return cext.boot_time()



AF_LINK = None

scputimes = namedtuple('scputimes', ['user', 'system', 'idle', 'iowait', 'nice'])


cpuloads = namedtuple('cpuloads', ['load1', 'load5', 'load15'])





#
# CPU
#

def cpu_times():
    ret = cext.per_cpu_times()
    return scputimes(*[sum(x) for x in zip(*ret)[1:]])


def per_cpu_times():
    """Return system per-CPU times as a list of named tuples."""
    cputimes = cext.per_cpu_times()
    ret = {}

    for x in cputimes:
        ret[x[0]] = scputimes(*x[1:])
    return ret

def cpu_count_logical():
    return cext.proc_cpu_num()


def cpu_stats():
    ctx_switches, interrupts, syscalls, traps = cext.cpu_stats()
    soft_interrupts = 0

    ret = cext.per_cpu_times()
    return _common.scpustats(*[sum(x) for x in zip(*ret)])
#    return _common.scpustats(ctx_switches, interrupts, soft_interrupts, syscalls)


def getloadavg():
    load1, load5, load15 = cext.cpu_load()
    return cpuloads(load1, load5, load15)


#
# MEMORY
#


# psutil.virtual_memory()
svmem = namedtuple('svmem', ['total', 'available', 'used', 'free', 'cached', 'pagesize'])

# psutil.Process.memory_info()
total_memory = cext.total_memory()

def memory_test():
    print cext.total_memory()
    print total_memory



def virtual_memory():
    total, available, used, free, cached, page = cext.virtual_memory()

    return svmem(total, available, used, free, cached, page)



def swap_memory():
    swap_total, swap_free, swap_used, sin, sout = cext.swap_memory()
    percent = float(swap_used) / float(swap_total) * 100
    return _common.sswap(swap_total, swap_used, swap_free, percent, sin, sout)

#
# DISK
#
#disk_io_counters = cext.disk_io_counters

def getDsfMap():
    if not os.path.isdir('/dev/disk'):
        return None

    ioscans = subprocess.Popen(['ioscan', '-m', 'dsf'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdout, stderr = ioscans.communicate()
    dsfs= stdout.split('\n')
    device_map = {}
    rdisk_pattern = re.compile('/dev/rdisk')
    rdsk_pattern = re.compile('/dev/rdsk')
    for dsf in dsfs[2:]:
        columns = dsf.split()
        if len(columns) != 2:
            continue
        key = rdisk_pattern.sub('/dev/disk', columns[0])
        value = rdsk_pattern.sub('/dev/dsk', columns[1])
        device_map[key] = value
    return device_map


gDiskMap = None
def getDiskMap():
    global gDiskMap
    if gDiskMap != None:
        return gDiskMap

    lvmMap = getLvmMap
    diskMap = {}

    for vg, disks in lvmMap.items():
        for disk in disks:
            if disk not in diskMap:
                diskMap[disk] = None
            diskMap[disk].add(vg)
    diskMap = {disk: list(vgs) for disk, vgs in diskMap.items()}
    gDiskMap = diskMap
    return diskMap


gLvmMap = None
def getLvmMap():
    global gLvmMap
    if gLvmMap != None:
        return gLvmMap

    try:
        with open('/etc/lvmtab', 'rb') as lvmtabFile:
            lvm = lvmtabFile.read()
    except IOError:
        return None
    lvm = re.findall(r"[^\x00-\x1F\x7F-\xFF]{4,}", lvm)
    vg_map = {}
    current_vg = None
    vg_pattern = re.compile(r'/dev/vg\w+')
    pv_dsk_pattern = re.compile(r'/dev/dsk/\w+')
    pv_disk_pattern = re.compile(r'/dev/disk/disk\d+_p\d+')

    for line in lvm:
        line = line.strip()
        vg_match = vg_pattern.match(line)
        if vg_match:
            current_vg = vg_match.group(0)
            vg_map[current_vg] = []
            continue

        pv_disk_match = pv_disk_pattern.match(line)
        if pv_disk_match and current_vg:
            pv = pv_disk_match.group(0)
            vg_map[current_vg].append(pv)
            continue

        pv_dsk_match = pv_dsk_pattern.match(line)
        if pv_dsk_match and current_vg:
            pv = pv_dsk_match.group(0)
            vg_map[current_vg].append(pv)
            continue

    gLvmMap = vg_map
    return vg_map


def disk_io_counters_lv(perdisk=False):
    ioCounters = cext.fs_io_counters()
    vg_pattern = re.compile(r'/dev/vg\w+')
 
    vgIoMap = {}
    for k, v in ioCounters.items():
        keyArr = k.split('|')
        vg_match = vg_pattern.match(keyArr[1])
        if vg_match:
            currentVg = vg_match.group(0)
            if currentVg in vgIoMap:
                vgIoMap[currentVg] = [x + y for x, y in zip(vgIoMap[currentVg], v)]
            else:
                vgIoMap[currentVg] = v
    
    lvmMap = getLvmMap()
    diskMap = {}

    for k, values in lvmMap.items():
        if k not in vgIoMap:
            continue

        io = vgIoMap[k] 
        for v in values:
            if v in diskMap:
                diskMap[v] = [x + y for x, y in zip(diskMap[v], io)]
            else:
                diskMap[v] = io

    return diskMap



def fs_io_counters(perdisk=False):
    ioCounters = cext.fs_io_counters()
    infos = cext.disk_io_info()
    times = cext.disk_io_time()
    vg_map = getLvmMap()


    ioTimeMap = {}
    for k, v in times.items():
        try:
            ioTimeMap[infos[k]] = v
        except KeyError:
            continue

    ret = {}

    vg_pattern = re.compile(r'/dev/vg\w+')
    pv_dsk_pattern = re.compile(r'(/dev/dsk/c\d+t\d+d\d+)')
    pv_disk_pattern = re.compile(r'(/dev/disk/disk\d+)')
    
    for k, v in ioCounters.items():
        keyArr = k.split('|')
        vg_match = vg_pattern.match(keyArr[1])
        pvs = []
        if vg_match:
            current_vg = vg_match.group(0)
            if current_vg in vg_map:
                pvs = vg_map[current_vg]

        times = {}
        for pv in pvs:
            pv_disk_match = pv_disk_pattern.match(pv)
            if pv_disk_match:
                pv_match = pv_disk_match.group(1)
                if pv_match in ioTimeMap:
                    times[pv_match] = ioTimeMap[pv_match]
                continue

            pv_dsk_match = pv_dsk_pattern.match(pv)
            if pv_dsk_match:
                pv_match = pv_dsk_match.group(1)
                if pv_match in ioTimeMap:
                    times[pv_match] = ioTimeMap[pv_match]
                continue
            
        ret[keyArr[0]] = sdiskio(v[0], v[1], v[2], v[3], 0, 0, times)
    return ret


def net_io_counters(pernic=False):
    ret = cext.net_io_counters()
    return ret

##
def disk_io_counters_hpux(perdisk=False):
    ret = cext.disk_io_counters()
    return ret

#
#
#


def wrap_exceptions(fun):
    """Call callable into a try/except clause and translate ENOENT,
    EACCES and EPERM in NoSuchProcess or AccessDenied exceptions.
    """
    @functools.wraps(fun)
    def wrapper(self, *args, **kwargs):
        try:
            return fun(self, *args, **kwargs)
        except (FileNotFoundError, ProcessLookupError):
            # ENOENT (no such file or directory) gets raised on open().
            # ESRCH (no such process) can get raised on read() if
            # process is gone in meantime.
            if not pid_exists(self.pid):
                raise NoSuchProcess(self.pid, self._name)
            else:
                raise ZombieProcess(self.pid, self._name, self._ppid)
        except PermissionError:
            raise AccessDenied(self.pid, self._name)
        except RuntimeError:
            raise NoSuchProcess(self.pid, self._name)

    return wrapper



def pids():
    """Returns a list of PIDs currently running on the system."""
    process = subprocess.Popen(['ps', '-e'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdout, stderr = process.communicate()
    processes = stdout.decode('utf-8').split('\n')

    pid_list = []

    for process in processes[1:]:
        if process.strip():
            pid = int(process.split()[0])
            if pid == 0: continue
            pid_list.append(pid)

    return pid_list



kinfo_proc_map = dict(
        pid=0,
        ppid=1,
        utime=2,
        stime=3,
        pcpu=4,
        rss=5,
        totalIO=6,
        cmdline=7,
        username=8,
        status=9,
        starttime=10,
        inblock=11,
        outblock=12,
        cputickstotal=13,
)

pcputimes = namedtuple('pcputimes', ['user', 'system', 'pcpu', 'cputickstotal'])
pmem = namedtuple('pmem', ['rss'])

pio = namedtuple('pio', ['read_count', 'write_count',
                         'read_bytes', 'write_bytes',
                         'total_bytes'])

PROC_STATUSES = {
    cext.SSLEEP: _common.STATUS_SLEEPING,
    cext.SRUN: _common.STATUS_RUNNING,
    cext.SSTOP: _common.STATUS_STOPPED,
    cext.SZOMB: _common.STATUS_ZOMBIE,
    cext.SIDL: _common.STATUS_IDLE,
    cext.SOTHER: _common.STATUS_RUNNING
}


class Process:
    """Wrapper class around underlying C implementation."""

    __slots__ = ["pid", "_name", "_ppid", "_cache"]

    def __init__(self, pid):
        self.pid = pid
        self._name = None
        self._ppid = None

    @wrap_exceptions
    def create_time(self):
        return cext.proc_create_time(self.pid)

    @wrap_exceptions
    @memoize_when_activated
    def oneshot_info(self):
        ret = cext.proc_oneshot_info(self.pid)
        return ret

    def oneshot_enter(self):
        self.oneshot_info.cache_activate(self)

    def oneshot_exit(self):
        self.oneshot_info.cache_deactivate(self)

    @wrap_exceptions
    def ppid(self):
        self._ppid = self.oneshot_info()[kinfo_proc_map['ppid']]
        return self._ppid

    @wrap_exceptions
    def name(self):
        cmd = self.cmdline()
        if len(cmd) == 0:
            return ""
        exe = cmd[0]
        return exe.split('/')[-1]


    @wrap_exceptions
    def exe(self):
        return self.name()

    @wrap_exceptions
    def cmdline(self):
        cmd = self.oneshot_info()[kinfo_proc_map['cmdline']]
        if cmd == "":
            return []
        return cmd.split()

    @wrap_exceptions
    def status(self):
        code = self.oneshot_info()[kinfo_proc_map['status']]
        return PROC_STATUSES.get(code, '?')

    @wrap_exceptions
    def username(self):

        return self.oneshot_info()[kinfo_proc_map['username']]

    @wrap_exceptions
    def cwd(self):
        return "cwd"
    
    @wrap_exceptions
    def nice_get(self):
        return "nice_get"
    
    @wrap_exceptions
    def nice_set(self):
        return "nice_set"

    @wrap_exceptions
    def uids(self):
        return "uids"

    @wrap_exceptions
    def gids(self):
        return "gids"

    @wrap_exceptions
    def memory_info(self):
        meminfo = (self.oneshot_info()[kinfo_proc_map['rss']])
        return pmem(meminfo)

    @wrap_exceptions
    def cpu_times(self):
        v = self.oneshot_info()
        times = (v[kinfo_proc_map['utime']], v[kinfo_proc_map['stime']], v[kinfo_proc_map['pcpu']], v[kinfo_proc_map['cputickstotal']])
        return pcputimes(*times)

    @wrap_exceptions
    def io_counters(self):
        v = self.oneshot_info()
        return pio(0, 0, v[kinfo_proc_map['inblock']], v[kinfo_proc_map['outblock']], v[kinfo_proc_map['totalIO']])


    @wrap_exceptions
    def open_files(self):
        rets = cext.proc_open_file(self.pid)
        retlist = []
        for path, fd in rets.items():
            retlist.append(_common.popenfile(path, int(fd)))
        return retlist 
        



def cpu_stats_detail():
    ret = cext.cpu_stats_detail()
    return ret

def virtual_memory_detail():
    ret = cext.virtual_memory_detail()
    return ret

def proc_total_info():
    ret = cext.proc_total_info()
    return ret

def proc_detail_info():
    ret = cext.proc_detail_info()
    return ret





from whatap.value.summary_value import SummaryValue
from whatap.value.value_enum import ValueEnum


class DoubleSummary(SummaryValue):
    def __init__(self):
        super(DoubleSummary. self).__init__()
        self.sum = 0
        self.count = 0
        self.min = 0
        self.max = 0

    def getValueType(self):
        return ValueEnum.DOUBLE_SUMMARY

    def write(self, dout):
        dout.writeDouble(self.sum)
        dout.writeInt(self.count)
        dout.writeDouble(self.min)
        dout.writeDouble(self.max)

    def read(self, din):
        self.sum = din.readDouble()
        self.count = din.readInt()
        self.min = din.readDouble()
        self.max = din.readDouble()
        return self

    def toObject(self):
        return self

    def addCount(self):
        self.count += 1

    def add(self, value):
        if not value:
            return self

        if isinstance(value, int):
            if not self.count:
                self.sum = value
                self.count = 1
                self.max = value
                self.min = value
            else:
                self.sum += value
                self.count += 1
                self.max = max(self.max, value)
                self.min = min(self.min, value)

        elif isinstance(value, SummaryValue):
            if not value.getCount():
                return self

            self.count += value.getCount()
            self.sum += value.doubleSum()
            self.min = min(self.min, value.doubleMin())
            self.max = max(self.max, value.doubleMax())
        return self

    def longSum(self):
        return self.sum

    def longMin(self):
        return self.min

    def longMax(self):
        return self.max

    def longAvg(self):
        if not self.count:
            return 0
        else:
            return self.sum / self.count

    def doubleSum(self):
        return self.sum

    def doubleMin(self):
        return self.min

    def doubleMax(self):
        return self.max

    def doubleAvg(self):
        if not self.count:
            return 0
        else:
            return self.sum / self.count

    def getCount(self):
        return self.count



import os
import sys
import ssl
import pprint
import urllib
import urlparse
# Rename HTTPServer to _HTTPServer so as to avoid confusion with HTTPSServer.
from BaseHTTPServer import HTTPServer as _HTTPServer, BaseHTTPRequestHandler
from SimpleHTTPServer import SimpleHTTPRequestHandler

from test import test_support as support
threading = support.import_module("threading")

here = os.path.dirname(__file__)

HOST = support.HOST
CERTFILE = os.path.join(here, 'keycert.pem')

# This one's based on HTTPServer, which is based on SocketServer

class HTTPSServer(_HTTPServer):

    def __init__(self, server_address, handler_class, context):
        _HTTPServer.__init__(self, server_address, handler_class)
        self.context = context

    def __str__(self):
        return ('<%s %s:%s>' %
                (self.__class__.__name__,
                 self.server_name,
                 self.server_port))

    def get_request(self):
        # override this to wrap socket with SSL
        try:
            sock, addr = self.socket.accept()
            sslconn = self.context.wrap_socket(sock, server_side=True)
        except OSError as e:
            # socket errors are silenced by the caller, print them here
            if support.verbose:
                sys.stderr.write("Got an error:\n%s\n" % e)
            raise
        return sslconn, addr

class RootedHTTPRequestHandler(SimpleHTTPRequestHandler):
    # need to override translate_path to get a known root,
    # instead of using os.curdir, since the test could be
    # run from anywhere

    server_version = "TestHTTPS/1.0"
    root = here
    # Avoid hanging when a request gets interrupted by the client
    timeout = 5

    def translate_path(self, path):
        """Translate a /-separated PATH to the local filename syntax.

        Components that mean special things to the local file system
        (e.g. drive or directory names) are ignored.  (XXX They should
        probably be diagnosed.)

        """
        # abandon query parameters
        path = urlparse.urlparse(path)[2]
        path = os.path.normpath(urllib.unquote(path))
        words = path.split('/')
        words = filter(None, words)
        path = self.root
        for word in words:
            drive, word = os.path.splitdrive(word)
            head, word = os.path.split(word)
            path = os.path.join(path, word)
        return path

    def log_message(self, format, *args):
        # we override this to suppress logging unless "verbose"
        if support.verbose:
            sys.stdout.write(" server (%s:%d %s):\n   [%s] %s\n" %
                             (self.server.server_address,
                              self.server.server_port,
                              self.request.cipher(),
                              self.log_date_time_string(),
                              format%args))


class StatsRequestHandler(BaseHTTPRequestHandler):
    """Example HTTP request handler which returns SSL statistics on GET
    requests.
    """

    server_version = "StatsHTTPS/1.0"

    def do_GET(self, send_body=True):
        """Serve a GET request."""
        sock = self.rfile.raw._sock
        context = sock.context
        stats = {
            'session_cache': context.session_stats(),
            'cipher': sock.cipher(),
            'compression': sock.compression(),
            }
        body = pprint.pformat(stats)
        body = body.encode('utf-8')
        self.send_response(200)
        self.send_header("Content-type", "text/plain; charset=utf-8")
        self.send_header("Content-Length", str(len(body)))
        self.end_headers()
        if send_body:
            self.wfile.write(body)

    def do_HEAD(self):
        """Serve a HEAD request."""
        self.do_GET(send_body=False)

    def log_request(self, format, *args):
        if support.verbose:
            BaseHTTPRequestHandler.log_request(self, format, *args)


class HTTPSServerThread(threading.Thread):

    def __init__(self, context, host=HOST, handler_class=None):
        self.flag = None
        self.server = HTTPSServer((host, 0),
                                  handler_class or RootedHTTPRequestHandler,
                                  context)
        self.port = self.server.server_port
        threading.Thread.__init__(self)
        self.daemon = True

    def __str__(self):
        return "<%s %s>" % (self.__class__.__name__, self.server)

    def start(self, flag=None):
        self.flag = flag
        threading.Thread.start(self)

    def run(self):
        if self.flag:
            self.flag.set()
        try:
            self.server.serve_forever(0.05)
        finally:
            self.server.server_close()

    def stop(self):
        self.server.shutdown()


def make_https_server(case, context=None, certfile=CERTFILE,
                      host=HOST, handler_class=None):
    if context is None:
        context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
    # We assume the certfile contains both private key and certificate
    context.load_cert_chain(certfile)
    server = HTTPSServerThread(context, host, handler_class)
    flag = threading.Event()
    server.start(flag)
    flag.wait()
    def cleanup():
        if support.verbose:
            sys.stdout.write('stopping HTTPS server\n')
        server.stop()
        if support.verbose:
            sys.stdout.write('joining HTTPS thread\n')
        server.join()
    case.addCleanup(cleanup)
    return server


if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(
        description='Run a test HTTPS server. '
                    'By default, the current directory is served.')
    parser.add_argument('-p', '--port', type=int, default=4433,
                        help='port to listen on (default: %(default)s)')
    parser.add_argument('-q', '--quiet', dest='verbose', default=True,
                        action='store_false', help='be less verbose')
    parser.add_argument('-s', '--stats', dest='use_stats_handler', default=False,
                        action='store_true', help='always return stats page')
    parser.add_argument('--curve-name', dest='curve_name', type=str,
                        action='store',
                        help='curve name for EC-based Diffie-Hellman')
    parser.add_argument('--ciphers', dest='ciphers', type=str,
                        help='allowed cipher list')
    parser.add_argument('--dh', dest='dh_file', type=str, action='store',
                        help='PEM file containing DH parameters')
    args = parser.parse_args()

    support.verbose = args.verbose
    if args.use_stats_handler:
        handler_class = StatsRequestHandler
    else:
        handler_class = RootedHTTPRequestHandler
        handler_class.root = os.getcwd()
    context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
    context.load_cert_chain(CERTFILE)
    if args.curve_name:
        context.set_ecdh_curve(args.curve_name)
    if args.dh_file:
        context.load_dh_params(args.dh_file)
    if args.ciphers:
        context.set_ciphers(args.ciphers)

    server = HTTPSServer(("", args.port), handler_class, context)
    if args.verbose:
        print("Listening on https://localhost:{0.port}".format(args))
    server.serve_forever(0.1)



# Test packages (dotted-name import)

import sys
import os
import tempfile
import textwrap
import unittest
from test import test_support


# Helpers to create and destroy hierarchies.

def cleanout(root):
    names = os.listdir(root)
    for name in names:
        fullname = os.path.join(root, name)
        if os.path.isdir(fullname) and not os.path.islink(fullname):
            cleanout(fullname)
        else:
            os.remove(fullname)
    os.rmdir(root)

def fixdir(lst):
    if "__builtins__" in lst:
        lst.remove("__builtins__")
    return lst


# XXX Things to test
#
# import package without __init__
# import package with __init__
# __init__ importing submodule
# __init__ importing global module
# __init__ defining variables
# submodule importing other submodule
# submodule importing global module
# submodule import submodule via global name
# from package import submodule
# from package import subpackage
# from package import variable (defined in __init__)
# from package import * (defined in __init__)


class Test(unittest.TestCase):

    def setUp(self):
        self.root = None
        self.pkgname = None
        self.syspath = list(sys.path)

    def tearDown(self):
        sys.path[:] = self.syspath
        if self.root: # Only clean if the test was actually run
            cleanout(self.root)

        # delete all modules concerning the tested hierarchy
        if self.pkgname:
            modules = [name for name in sys.modules
                       if self.pkgname in name.split('.')]
            for name in modules:
                del sys.modules[name]

    def run_code(self, code):
        exec(textwrap.dedent(code), globals(), {"self": self})

    def mkhier(self, descr):
        root = tempfile.mkdtemp()
        sys.path.insert(0, root)
        if not os.path.isdir(root):
            os.mkdir(root)
        for name, contents in descr:
            comps = name.split()
            fullname = root
            for c in comps:
                fullname = os.path.join(fullname, c)
            if contents is None:
                os.mkdir(fullname)
            else:
                f = open(fullname, "w")
                f.write(contents)
                if contents and contents[-1] != '\n':
                    f.write('\n')
                f.close()
        self.root = root
        # package name is the name of the first item
        self.pkgname = descr[0][0]

    def test_1(self):
        hier = [("t1", None), ("t1 __init__"+os.extsep+"py", "")]
        self.mkhier(hier)
        import t1

    def test_2(self):
        hier = [
         ("t2", None),
         ("t2 __init__"+os.extsep+"py", "'doc for t2'"),
         ("t2 sub", None),
         ("t2 sub __init__"+os.extsep+"py", ""),
         ("t2 sub subsub", None),
         ("t2 sub subsub __init__"+os.extsep+"py", "spam = 1"),
        ]
        self.mkhier(hier)

        import t2.sub
        import t2.sub.subsub
        self.assertEqual(t2.__name__, "t2")
        self.assertEqual(t2.sub.__name__, "t2.sub")
        self.assertEqual(t2.sub.subsub.__name__, "t2.sub.subsub")

        # This exec crap is needed because Py3k forbids 'import *' outside
        # of module-scope and __import__() is insufficient for what we need.
        s = """
            import t2
            from t2 import *
            self.assertEqual(dir(), ['self', 'sub', 't2'])
            """
        self.run_code(s)

        from t2 import sub
        from t2.sub import subsub
        from t2.sub.subsub import spam
        self.assertEqual(sub.__name__, "t2.sub")
        self.assertEqual(subsub.__name__, "t2.sub.subsub")
        self.assertEqual(sub.subsub.__name__, "t2.sub.subsub")
        for name in ['spam', 'sub', 'subsub', 't2']:
            self.assertTrue(locals()["name"], "Failed to import %s" % name)

        import t2.sub
        import t2.sub.subsub
        self.assertEqual(t2.__name__, "t2")
        self.assertEqual(t2.sub.__name__, "t2.sub")
        self.assertEqual(t2.sub.subsub.__name__, "t2.sub.subsub")

        s = """
            from t2 import *
            self.assertTrue(dir(), ['self', 'sub'])
            """
        self.run_code(s)

    def test_3(self):
        hier = [
                ("t3", None),
                ("t3 __init__"+os.extsep+"py", ""),
                ("t3 sub", None),
                ("t3 sub __init__"+os.extsep+"py", ""),
                ("t3 sub subsub", None),
                ("t3 sub subsub __init__"+os.extsep+"py", "spam = 1"),
               ]
        self.mkhier(hier)

        import t3.sub.subsub
        self.assertEqual(t3.__name__, "t3")
        self.assertEqual(t3.sub.__name__, "t3.sub")
        self.assertEqual(t3.sub.subsub.__name__, "t3.sub.subsub")

    def test_4(self):
        hier = [
        ("t4.py", "raise RuntimeError('Shouldnt load t4.py')"),
        ("t4", None),
        ("t4 __init__"+os.extsep+"py", ""),
        ("t4 sub.py", "raise RuntimeError('Shouldnt load sub.py')"),
        ("t4 sub", None),
        ("t4 sub __init__"+os.extsep+"py", ""),
        ("t4 sub subsub"+os.extsep+"py",
         "raise RuntimeError('Shouldnt load subsub.py')"),
        ("t4 sub subsub", None),
        ("t4 sub subsub __init__"+os.extsep+"py", "spam = 1"),
               ]
        self.mkhier(hier)

        s = """
            from t4.sub.subsub import *
            self.assertEqual(spam, 1)
            """
        self.run_code(s)

    def test_5(self):
        hier = [
        ("t5", None),
        ("t5 __init__"+os.extsep+"py", "import t5.foo"),
        ("t5 string"+os.extsep+"py", "spam = 1"),
        ("t5 foo"+os.extsep+"py",
         "from . import string; assert string.spam == 1"),
         ]
        self.mkhier(hier)

        import t5
        s = """
            from t5 import *
            self.assertEqual(dir(), ['foo', 'self', 'string', 't5'])
            """
        self.run_code(s)

        import t5
        self.assertEqual(fixdir(dir(t5)),
                         ['__doc__', '__file__', '__name__',
                          '__package__', '__path__', 'foo', 'string', 't5'])
        self.assertEqual(fixdir(dir(t5.foo)),
                         ['__doc__', '__file__', '__name__', '__package__',
                          'string'])
        self.assertEqual(fixdir(dir(t5.string)),
                         ['__doc__', '__file__', '__name__','__package__',
                          'spam'])

    def test_6(self):
        hier = [
                ("t6", None),
                ("t6 __init__"+os.extsep+"py",
                 "__all__ = ['spam', 'ham', 'eggs']"),
                ("t6 spam"+os.extsep+"py", ""),
                ("t6 ham"+os.extsep+"py", ""),
                ("t6 eggs"+os.extsep+"py", ""),
               ]
        self.mkhier(hier)

        import t6
        self.assertEqual(fixdir(dir(t6)),
                         ['__all__', '__doc__', '__file__',
                          '__name__', '__package__', '__path__'])
        s = """
            import t6
            from t6 import *
            self.assertEqual(fixdir(dir(t6)),
                             ['__all__', '__doc__', '__file__',
                              '__name__', '__package__', '__path__',
                              'eggs', 'ham', 'spam'])
            self.assertEqual(dir(), ['eggs', 'ham', 'self', 'spam', 't6'])
            """
        self.run_code(s)

    def test_7(self):
        hier = [
                ("t7", None),
                ("t7"+os.extsep+"py", ""),
                ("t7 __init__"+os.extsep+"py", ""),
                ("t7 sub"+os.extsep+"py",
                 "raise RuntimeError('Shouldnt load sub.py')"),
                ("t7 sub", None),
                ("t7 sub __init__"+os.extsep+"py", ""),
                ("t7 sub "+os.extsep+"py",
                 "raise RuntimeError('Shouldnt load subsub.py')"),
                ("t7 sub subsub", None),
                ("t7 sub subsub __init__"+os.extsep+"py",
                 "spam = 1"),
               ]
        self.mkhier(hier)


        t7, sub, subsub = None, None, None
        import t7 as tas
        self.assertEqual(fixdir(dir(tas)),
                         ['__doc__', '__file__', '__name__',
                          '__package__', '__path__'])
        self.assertFalse(t7)
        from t7 import sub as subpar
        self.assertEqual(fixdir(dir(subpar)),
                         ['__doc__', '__file__', '__name__',
                          '__package__', '__path__'])
        self.assertFalse(t7)
        self.assertFalse(sub)
        from t7.sub import subsub as subsubsub
        self.assertEqual(fixdir(dir(subsubsub)),
                         ['__doc__', '__file__', '__name__',
                         '__package__', '__path__', 'spam'])
        self.assertFalse(t7)
        self.assertFalse(sub)
        self.assertFalse(subsub)
        from t7.sub.subsub import spam as ham
        self.assertEqual(ham, 1)
        self.assertFalse(t7)
        self.assertFalse(sub)
        self.assertFalse(subsub)

    @unittest.skipIf(sys.flags.optimize >= 2,
                     "Docstrings are omitted with -O2 and above")
    def test_8(self):
        hier = [
                ("t8", None),
                ("t8 __init__"+os.extsep+"py", "'doc for t8'"),
               ]
        self.mkhier(hier)

        import t8
        self.assertEqual(t8.__doc__, "doc for t8")

def test_main():
    test_support.run_unittest(__name__)


if __name__ == "__main__":
    test_main()



import unittest

from Tkinter import Variable, StringVar, IntVar, DoubleVar, BooleanVar, Tcl, TclError


class TestBase(unittest.TestCase):

    def setUp(self):
        self.root = Tcl()

    def tearDown(self):
        del self.root


class TestVariable(TestBase):

    def info_exists(self, *args):
        return self.root.getboolean(self.root.call("info", "exists", *args))

    def test_default(self):
        v = Variable(self.root)
        self.assertEqual("", v.get())
        self.assertRegexpMatches(str(v), r"^PY_VAR(\d+)$")

    def test_name_and_value(self):
        v = Variable(self.root, "sample string", "varname")
        self.assertEqual("sample string", v.get())
        self.assertEqual("varname", str(v))

    def test___del__(self):
        self.assertFalse(self.info_exists("varname"))
        v = Variable(self.root, "sample string", "varname")
        self.assertTrue(self.info_exists("varname"))
        del v
        self.assertFalse(self.info_exists("varname"))

    def test_dont_unset_not_existing(self):
        self.assertFalse(self.info_exists("varname"))
        v1 = Variable(self.root, name="name")
        v2 = Variable(self.root, name="name")
        del v1
        self.assertFalse(self.info_exists("name"))
        # shouldn't raise exception
        del v2
        self.assertFalse(self.info_exists("name"))

    def test___eq__(self):
        # values doesn't matter, only class and name are checked
        v1 = Variable(self.root, name="abc")
        v2 = Variable(self.root, name="abc")
        self.assertEqual(v1, v2)

        v3 = Variable(self.root, name="abc")
        v4 = StringVar(self.root, name="abc")
        self.assertNotEqual(v3, v4)

    def test_invalid_name(self):
        with self.assertRaises(TypeError):
            Variable(self.root, name=123)

    def test_null_in_name(self):
        with self.assertRaises(ValueError):
            Variable(self.root, name='var\x00name')
        with self.assertRaises(ValueError):
            self.root.globalsetvar('var\x00name', "value")
        with self.assertRaises(ValueError):
            self.root.setvar('var\x00name', "value")


class TestStringVar(TestBase):

    def test_default(self):
        v = StringVar(self.root)
        self.assertEqual("", v.get())

    def test_get(self):
        v = StringVar(self.root, "abc", "name")
        self.assertEqual("abc", v.get())
        self.root.globalsetvar("name", "value")
        self.assertEqual("value", v.get())

    def test_get_null(self):
        v = StringVar(self.root, "abc\x00def", "name")
        self.assertEqual("abc\x00def", v.get())
        self.root.globalsetvar("name", "val\x00ue")
        self.assertEqual("val\x00ue", v.get())


class TestIntVar(TestBase):

    def test_default(self):
        v = IntVar(self.root)
        self.assertEqual(0, v.get())

    def test_get(self):
        v = IntVar(self.root, 123, "name")
        self.assertEqual(123, v.get())
        self.root.globalsetvar("name", "345")
        self.assertEqual(345, v.get())

    def test_invalid_value(self):
        v = IntVar(self.root, name="name")
        self.root.globalsetvar("name", "value")
        with self.assertRaises(ValueError):
            v.get()
        self.root.globalsetvar("name", "345.0")
        with self.assertRaises(ValueError):
            v.get()


class TestDoubleVar(TestBase):

    def test_default(self):
        v = DoubleVar(self.root)
        self.assertEqual(0.0, v.get())

    def test_get(self):
        v = DoubleVar(self.root, 1.23, "name")
        self.assertAlmostEqual(1.23, v.get())
        self.root.globalsetvar("name", "3.45")
        self.assertAlmostEqual(3.45, v.get())

    def test_get_from_int(self):
        v = DoubleVar(self.root, 1.23, "name")
        self.assertAlmostEqual(1.23, v.get())
        self.root.globalsetvar("name", "3.45")
        self.assertAlmostEqual(3.45, v.get())
        self.root.globalsetvar("name", "456")
        self.assertAlmostEqual(456, v.get())

    def test_invalid_value(self):
        v = DoubleVar(self.root, name="name")
        self.root.globalsetvar("name", "value")
        with self.assertRaises(ValueError):
            v.get()


class TestBooleanVar(TestBase):

    def test_default(self):
        v = BooleanVar(self.root)
        self.assertEqual(False, v.get())

    def test_get(self):
        v = BooleanVar(self.root, True, "name")
        self.assertAlmostEqual(True, v.get())
        self.root.globalsetvar("name", "0")
        self.assertAlmostEqual(False, v.get())

    def test_invalid_value_domain(self):
        v = BooleanVar(self.root, name="name")
        self.root.globalsetvar("name", "value")
        with self.assertRaises(TclError):
            v.get()
        self.root.globalsetvar("name", "1.0")
        with self.assertRaises(TclError):
            v.get()


tests_gui = (TestVariable, TestStringVar, TestIntVar,
             TestDoubleVar, TestBooleanVar)


if __name__ == "__main__":
    from test.support import run_unittest
    run_unittest(*tests_gui)



#
# test_codecmaps_tw.py
#   Codec mapping tests for ROC encodings
#

from test import test_support
from test import test_multibytecodec_support
import unittest

class TestBIG5Map(test_multibytecodec_support.TestBase_Mapping,
                  unittest.TestCase):
    encoding = 'big5'
    mapfileurl = 'http://www.pythontest.net/unicode/BIG5.TXT'

class TestCP950Map(test_multibytecodec_support.TestBase_Mapping,
                   unittest.TestCase):
    encoding = 'cp950'
    mapfileurl = 'http://www.pythontest.net/unicode/CP950.TXT'
    pass_enctest = [
        ('\xa2\xcc', u'\u5341'),
        ('\xa2\xce', u'\u5345'),
    ]

def test_main():
    test_support.run_unittest(__name__)

if __name__ == "__main__":
    test_main()



import whatap.util.logging_util as logging_util
import whatap.agent.conf.configure as config
import whatap.agent.secure.security_master as secure
import whatap.net.tcpsession as tcpsession
import whatap.net as net
from whatap.pack.pack import Pack
from whatap.io.data_inputx import DataInputX
from whatap.util.date_util import DateUtil as dateutil
import threading, time, Queue
import math

class __Receiver(threading.Thread):
    def __init__(self) :
        threading.Thread.__init__(self, name='__Receiver')
        self.setDaemon(True)
        self.__queue = Queue.Queue()
        self.__timeSynced = False

    def isTimeSynced(self):
        return self.__timeSynced
        
    def run(self):
        #logging_util.debug('receiver.run step -0.1')
        conf = config.GetConfig()
        secuMaster = secure.GetSecurityMaster()
        #logging_util.debug('receiver.run step -1')
        secuMaster.WaitForInit()
        #logging_util.debug('receiver.run step -2')
        secuSession = secure.GetSecuritySession()
        #logging_util.debug('receiver.run step -3')
        while True:
            try:
                #logging_util.debug('receiver.run step -4')
                session = tcpsession.GetTcpSession()
                while not session.isOpen():
                    session = tcpsession.GetTcpSession()
                    time.sleep(1)
                #logging_util.debug('receiver.run step -5')
                try:
                    out = session.Read()
                except Exception, e:
                    logging_util.error("[Receiver] session.Read failed, closing session: %s" % e)
                    session.Close()
                    time.sleep(1)
                    continue
                #print 'receiver incomming pack', out.code
                if out.code == net.NET_TIME_SYNC :
                    in_ = DataInputX(out.data)
                    prevAgentTime = in_.readLong()
                    serverTime = in_.readLong()
                    now = dateutil.now()
                    turnaroundTime = now - prevAgentTime
                    if turnaroundTime < 500 and math.fabs(float(serverTime-now)) > 1000 :
                        dateutil.setServerDelta(dateutil.getServerDelta()+serverTime - now)
                    self.__timeSynced = True
                    continue

                if conf.CypherLevel > 0 :
                    if out.transferKey != secuSession.transferKey :
                        continue
                    secureMask =  net.GetSecureMask(out.code)
                    if secureMask == net.NET_SECURE_HIDE:
                        if secuSession.cypher is not None:
                            out.data = secuSession.cypher.HideLong(out.data)

                    elif secureMask == net.NET_SECURE_CYPHER:
                        if secuSession.cypher is not None:
                            data_len = len(out.data) if out.data is not None else -1
                            try:
                                out.data = secuSession.cypher.Decrypt(out.data)
                            except Exception, ce:
                                logging_util.error(
                                    "[Receiver] Decrypt failed code=%s len=%d block=%s err=%s" %
                                    (out.code, data_len, secuSession.cypher.block, ce))
                                raise
                    else:
                        logging_util.info(
                            "[Receiver] dropping pack with no secure mask code=%s mask=%s len=%s" %
                            (out.code, secureMask, len(out.data) if out.data is not None else -1))
                        out.data = None
                #print 'receiver step -0.1'
                if out.data:
                    p = Pack.toPack(out.data)
                    #print 'receiver step -1'
                    self.__queue.put(p)

                time.sleep(0.001)
            except Exception, e:
                logging_util.logStack(e)
                time.sleep(0.001)

    def get(self):
        return self.__queue.get()

__receiver = __Receiver()
__receiver.start()

def getReceiver():
    global __receiver

    return __receiver



import simpleplugin
from simpleplugin.agent import Agent
import os, sys, time
from Queue import Queue
import whatap.value as whatapValue
import whatap.util.thread_util as thread_util
import whatap.util.logging_util as logging_util
from whatap.util.date_util import DateUtil as date_util
import whatap.agent.secure.security_master as secure
import whatap.agent.data.data_pack as data
from whatap.pack.tagcount_pack import TagCountPack
from whatap.pack.logsink_pack import LogSinkPack

def start():
	thread_util.async(_start)

agentFacade = None

def getAgent():
	global agentFacade
	if not agentFacade:
		raise Exception("agent not enabled")
	
	return agentFacade

def getInput(inputName):
	agentFacade = getAgent()
	
	return agentFacade.getInputToml(inputName)
	
def setInput(inputfile, inputToml ):
	# if !config.GetConfig().ExtensionUnlock {
	# 	err = fmt.Errorf("Modification failed. Extension read only mode")
	# 	return

	agentFacade = getAgent()
	
	agentFacade.setInputToml(inputfile, inputToml)
	
	return

def getAllInput():
	# if !config.GetConfig().ExtensionUnlock {
	# 	err = fmt.Errorf("Modification failed. Extension read only mode")
	# 	return
	
	agentFacade = getAgent()
	
	return agentFacade.getAllInput()

def deleteInput(inputfile ):
	# if !config.GetConfig().ExtensionUnlock {
	# 	err = fmt.Errorf("Modification failed. Extension read only mode")
	# 	return
	
	agentFacade = getAgent()
	
	agentFacade.deleteInput(inputfile)
	
	return


def _start():
	global agentFacade
	prefix = os.path.abspath(os.path.split(sys.argv[0])[0])
	pluginPrefix = os.path.join(prefix, "extension")
	if not os.path.exists(pluginPrefix):
		logging_util.debug("simpleplugin extension directory not found: {}".format(pluginPrefix))
		return
	
	try:
		agentFacade = Agent.newAgent(pluginPrefix)
	except Exception, e:
		logging_util.debug('simpleplugin start error: {}'.format(e))
		return

	src = Queue(maxsize=1000)
	agentFacade.startEx(src)

	listenForever(src)

def writeTagCount(m, pcode, oid, now, oname):
	p = TagCountPack(category=m.name)
	p.pcode = pcode
	p.oid = oid
	p.time = now
	for field in m.fieldList():
		p.fields.put(field.key, whatapValue.toValue(field.value))
	for tag in m.tagList():
		p.tags.put(tag.key, whatapValue.toValue(tag.value))
	
	p.putTag("oname", whatapValue.toValue(oname))

	data.SendHide(p)

def writeLog(m, pcode, oid, now, oname):
	#print("simplePluginWrapper writeLog ", m, pcode, oid, now, oname)
	p = LogSinkPack(category=m.name)
	p.pcode = pcode
	p.oid = oid
	p.time = now
	if m.fieldList():
		for field in m.fieldList():
			p.fields.put(field.key, whatapValue.toValue(field.value))
	if m.tagList():
		for tag in m.tagList():
			p.tags.put(tag.key, whatapValue.toValue(tag.value))
		
	p.tags.put("oname", whatapValue.toValue(oname))
	p.content = m.content
	p.line = m.line

	

	data.SendHide(p)


def listenForever(src):
	secure.GetSecurityMaster().WaitForInit()
	while True:
		secu = secure.GetSecurityMaster()
		pcode = secu.pcode
		oid = secu.oid
		oname = secu.oname
		# if not config.GetConfig().IsExtensionEnabled:
		# 	continue
		m = src.get()
		
		if not m:
			time.sleep(1)
			continue 
		now = date_util.now()
		if m and isinstance(m, simpleplugin.LogMetric):
			try:
				writeLog(m, pcode, oid, now, oname)
			except Exception, e:
				logging_util.debug("simplepluginWrapper writeLog error:{}".format(e))
		elif m and isinstance(m, simpleplugin.Metric):
			try:
				writeTagCount(m, pcode, oid, now, oname)
			except Exception, e:
				logging_util.debug("simplepluginWrapper writeTagCount error:{}".format(e))
			


from whatap.pack.pack import Pack
from whatap.pack.pack_enum import PackEnum
from whatap.io.data_outputx import DataOutputX
from whatap.value.map_value import MapValue
import whatap.value.value as value
from whatap.util.hash_util import HashUtil

from StringIO import StringIO

class TagCountPack(Pack):
    def __init__(self, category= None):
        Pack.__init__(self)

        self.category = category
        self.taghash = 0
        self.tags = MapValue()
        self.fields = MapValue()

    def getPackType(self):
        return PackEnum.PACK_TAGCOUNT

    def write(self, dout ):
        super(TagCountPack, self).write(dout)

        dout.writeByte(0)
        dout.writeText(self.category)
        if self.taghash == 0 and self.tags.size() > 0:
            tagIO = DataOutputX()
            value.writeValue(tagIO, self.tags)
            tagbytes = tagIO.toByteArray()
            self.taghash = HashUtil.hash64(tagbytes)
            dout.writeDecimal(self.taghash)
            dout.write(tagbytes)
        else:
            dout.writeDecimal(self.taghash)
            value.writeValue(dout, self.tags)

        value.writeValue(dout, self.fields)

    def put(self, key, value):
        if isinstance(value, int) or isinstance(value, long):
            self.fields.putLong(key, value)
        elif isinstance(value, float):
            self.fields.putFloat(key, value)
        elif isinstance(value, str):
            self.fields.putString(key, value)
        else:
            self.fields.put(key, value)

    def putTag(self, key, value):
        self.tags.putString(key, value)

    def read(self, din ):
        super(TagCountPack, self).read(din)
        din.readByte()
        self.category = din.readText()
        self.tags = value.readValue(din)
        self.fields = value.readValue(din)

    def size(self):
        return self.fields.size()

    def isempty(self):
        return self.fields.size() < 1

    def clear(self):
        self.fields.clear()



from whatap.io.data_outputx import DataOutputX
from whatap.io.data_inputx import DataInputX
from whatap.pack.pack import Pack
from whatap.pack.pack_enum import PackEnum



class TCPPortPerf :
	def __init__(self):
		self.port = None
		self.isAlive= None
		self.ip = None
		
	def write(self, out):
		dout = DataOutputX()
		dout.writeInt(self.port)
		dout.writeBoolean(self.isAlive)
		dout.writeInt(self.ip)
		out.writeBlob(dout.toByteArray())

	def read(self, inx):
		din = DataInputX( inx.readBlob())

		self.port = din.readInt()
		self.isAlive= din.readBoolean()
		self.ip= din.readInt()


class SMTCPPortPerfPack(Pack):
    def __init__(self):
        Pack.__init__(self)
        self.tcpPortPerf = []

    def getPackType(self):
        return PackEnum.PACK_SM_PORT_PERF

    def write(self, dout ):
        super(SMTCPPortPerfPack, self).write(dout)
        dout.writeDecimal(len(self.tcpPortPerf))
        for p in self.tcpPortPerf:
            p.write(dout)

    def read(self, dinx):
        din= DataInputX(dinx.readBlob())
        super(SMTCPPortPerfPack, self).read(din)
      
        cnt = din.readDecimal()
        for i in range(cnt):
            p = TCPPortPerf()
            p.read(din)
            self.tcpPortPerf.append(p)


import time

from whatap import logging
from whatap.util.linked_list import LinkedList


class RequestDoubleQueue(object):
    def __init__(self, capacity1, capacity2):
        self.capacity1 = capacity1
        self.capacity2 = capacity2
        self.queue1 = LinkedList()
        self.queue2 = LinkedList()

    def get(self):
        while self.queue1.size() <= 0 and self.queue2.size() <= 0:
            try:
                time.sleep(0.5)
            except Exception, e:
                logging.debug(e, extra={'id': 'WA510'}, exc_info=True)

        if self.queue1.size() > 0:
            return self.queue1.removeFirst()
        elif self.queue2.size() > 0:
            return self.queue2.removeFirst()

        return None

    def hasNext(self):
        if self.queue1.hasNext() or self.queue2.hasNext():
            return True
        return False

    def putForce1(self, o):
        return self.putForce(self.queue1, self.capacity1, o)

    def putForce2(self, o):
        return self.putForce(self.queue2, self.capacity2, o)

    def putForce(self, q, sz, o):
        if sz <= 0 or q.size() < self.capacity1:
            q.add(o)
            return True
        else:
            while q.size() >= sz:
                v = q.removeFirst()
                self.overflowed(v)
            q.add(o)
            return False

    def put1(self, o):
        return self.put(self.queue1, self.capacity1, o)

    def put2(self, o):
        return self.put(self.queue1, self.capacity2, o)

    def put(self, queue, capacity, o):
        if capacity <= 0 or queue.size() < capacity:
            queue.add(o)
            return True
        else:
            self.failed(o)
            return False

    def failed(self, v):
        return

    def overflowed(self, v):
        return
